from contextlib import contextmanager
import os.path
from tempfile import mkdtemp
from shutil import rmtree

from testtools import TestCase
from mock import (
    call,
    patch,
)
import yaml

import hooks


@contextmanager
def temp_dir():
    directory = mkdtemp()
    try:
        yield directory
    finally:
        rmtree(directory)


class HooksTest(TestCase):
    def test__get_key_file_location_empty(self):
        """No ssl_keylocation, expect None."""
        self.assertEqual(hooks._get_key_file_location(
            {"ssl_keylocation": None}), None)

    def test__get_key_file_location(self):
        """ssl_keylocation, expect correct path."""
        self.assertEqual(hooks._get_key_file_location(
            {"ssl_keylocation": "foo"}), "/etc/ssl/private/foo")

    def test__get_cert_file_location_empty(self):
        """No ssl_keylocation, expect None."""
        self.assertEqual(hooks._get_cert_file_location(
            {"ssl_certlocation": None}), None)

    def test__get_cert_file_location(self):
        """ssl_keylocation, expect correct path."""
        self.assertEqual(hooks._get_cert_file_location(
            {"ssl_certlocation": "foo"}), "/etc/ssl/certs/foo")

    def test__get_chain_file_location_empty(self):
        """No ssl_keylocation, expect None."""
        self.assertEqual(hooks._get_chain_file_location(
            {"ssl_chainlocation": None}), None)

    def test__get_chain_file_location(self):
        """ssl_keylocation, expect correct path."""
        self.assertEqual(hooks._get_chain_file_location(
            {"ssl_chainlocation": "foo"}), "/etc/ssl/certs/foo")


class TestEnsurePorts(TestCase):

    def test_ensure_ports(self):
        with patch('hooks.open_port') as hop_mock:
            with patch('hooks.set_open_ports') as hsop_mock:
                with patch('hooks.get_open_ports', return_value=[]):
                    hooks.ensure_ports([80, 81])
        self.assertEqual(hop_mock.mock_calls, [call(80), call(81)])
        hsop_mock.assert_called_once_with([80, 81])

    def test_idempotent(self):
        with patch('hooks.get_open_ports', return_value=[81, 82]):
            with patch('hooks.open_port') as hop_mock:
                with patch('hooks.set_open_ports') as hsop_mock:
                    hooks.ensure_ports([81, 83, 82])
        self.assertEqual(hop_mock.mock_calls, [call(83)])
        hsop_mock.assert_called_once_with([81, 82, 83])

    def test_closes_unneeded(self):
        with patch('hooks.get_open_ports', return_value=[81, 82, 83, 84]):
            with patch('hooks.close_port') as hcp_mock:
                with patch('hooks.set_open_ports') as hsop_mock:
                    hooks.ensure_ports([81, 82])
        self.assertEqual(hcp_mock.mock_calls, [call(83), call(84)])
        hsop_mock.assert_called_once_with([81, 82])


class TestGetSetOpenPorts(TestCase):

    def test_get_open_ports(self):
        with temp_dir() as charm_dir:
            with patch('os.environ', {'CHARM_DIR': charm_dir}):
                self.assertEqual(hooks.get_open_ports(), [])
                with open(os.path.join(charm_dir, 'ports.yaml'), 'w') as pfile:
                    yaml.safe_dump([47, 67], pfile)
                self.assertEqual(hooks.get_open_ports(), [47, 67])

    def test_set_open_ports(self):
        with temp_dir() as charm_dir:
            with patch('os.environ', {'CHARM_DIR': charm_dir}):
                hooks.set_open_ports([44, 23])
            with open(os.path.join(charm_dir, 'ports.yaml')) as pfile:
                self.assertEqual(yaml.safe_load(pfile), [44, 23])
